function [exitFlag exitMsg netStruct] = ba_networkUtilityMaximization(netStruct, algorithmParameters, net2planParameters)

	% This algorithm sets the routing as shortest path in number of hops.
	% Then, creates a demand between every node pair. Then, the amount of
	% traffic r_d offered by each demand d is calculated as follows.
	% Each demand has an utility function U_alpha given by:
    % U_alpha(r_d) = r_d if alpha = 0
    % U_alpha(r_d) = log(r_d) if alpha = 1
    % U_alpha(r_d) = r_d^(1-alpha) / (1-alpha), otherwise
    % The decision variables r_d are the bandwidth granted to each demand d
    % The network utility is the sum of the utilities of the demands. The
    % objetive function is to maximize the network utility. It is known that 
    % a bandwidth sharing that maximimes network utility with utility 
    % function U_alpha, is a sharing that is alpha-fair.
    %
    % The offeredTrafficInErlangs for each demand d is set to r_d. 
    %
    % It requires CVX solver installed and running.
    %
    % PARAM: alpha | 2 | Parameter to define utility function (alpha>=0)

    % Sanity checks
    alpha = str2double(algorithmParameters.alpha);
    assert (alpha >= 0 , '"alpha" must be a non-negative number');
    assert(isCvxInstalledAndRunning(), 'CVX solver must be installed and running');
    assert(isfield(netStruct, 'linkTable'), 'A set of links is required');

    % initialize some variables
    E = size(netStruct.linkTable, 1);
    N = max (netStruct.linkTable (:));
    u_e = netStruct.linkCapacityInErlangs;
    
    % create a demand table with one demand per each node pair
    [netStruct.demandTable netStruct.offeredTrafficInErlangs] = trafficMatrix2demandTable(ones (N,N) - eye(N,N));
    D = size(netStruct.demandTable, 1);
    
    % create a routing which is shortest path
    [exitFlag exitMsg netStruct] = fa_shortestPath_numHops (netStruct, algorithmParameters, net2planParameters);
    if (exitFlag ~= 0), return; end
    
    % compute some aux variables for later
    delta_pe = seqLinksPerPath2linkOccupancyPerPath(netStruct.pathList, E);
    
    cvx_begin

        cvx_quiet(true);

        % Variables
        variables r_d(1,D); % bandwidth granted to each demand

        % Objective function
        if alpha == 0
            maximize(sum(r_d))
        elseif alpha == 1
            maximize(sum(log(r_d)))
        else
            maximize(((1-alpha)^-1) * sum(pow_p(r_d , 1-alpha)))
        end

        % Constraints
        subject to

            % traffic in each link does not exceed capacity
            r_d * netStruct.routingMatrix * delta_pe <= u_e; 
            
            % Non-negativity constraints (D)
            r_d >= 0;   

    cvx_end
    
    if strcmp(cvx_status, 'Solved') || strcmp(cvx_status, 'Inaccurate/Solved')

        netStruct.offeredTrafficInErlangs = r_d;
        netStruct.demandAttributes = cellfun(@(x) struct, cell(1,D), 'UniformOutput', false);

        exitFlag = 0;
        exitMsg = 'OK';

    else

        exitFlag = -1;
        exitMsg = cvx_status;
        netStruct = [];

    end
        
 end

