function trafficMatrix = demandTable2trafficMatrix(demandTable, offeredTrafficPerDemand, N)

    % Function to convert a Dx2 demand table in a NxN traffic matrix where
    % each entry (i,j) represents the units of traffic from i to j.
    %
    % If it exists more than one demand per node pair, this information is
    % lost in the traffic matrix since they are grouped in one entry.
    %
    % Syntax: trafficMatrix = demandTable2trafficMatrix(demandTable,
    % offeredTrafficPerDemand, N)
    %
    % Input parameters:
    % - demandTable (Dx2): Table in which each row represents the ingress
    %   and egress node of the demand
    % - offeredTrafficPerDemand (1xD): Row vector in which each entry
    %   represents the offered traffic for that demand
    % - N: Number of nodes in the whole network
    %
    % If demandTable = [], trafficMatrix will be a NxN zero matrix
    %
    % Output parameters:
    % - trafficMatrix (NxN): Square matrix in which each entry (i,j) in
    %   shows the units of traffic offered from i to j
    
	% Sanity checks
	assert(numel(N) == 1 && isfinite(N) && N > 0 && N == floor(N), ...
		'N must be an integer number greater than 0')
	
	if isempty(demandTable)
		trafficMatrix = zeros(N, N);
	else
		assert(isDemandTable(demandTable, N), 'A valid set of demand is required');
		assert(size(demandTable, 1) == size(offeredTrafficPerDemand, 2) && ...
			size(offeredTrafficPerDemand, 1) == 1 && ...
			all(isfinite(offeredTrafficPerDemand(:))) && ...
			all(offeredTrafficPerDemand(:) >= 0), ...
			'offeredTrafficPerDemand must be a 1xD non-negative vector');
		
		% Perform traffic matrix computation
		trafficMatrix = accumarray(demandTable, offeredTrafficPerDemand, [N N]);
	end

end